"""
Pluggable Auth Service (PAS) Integration

pas integration with teamspace focuses on group and property based
aspects of pas.

as regards the group aspects there are two obvious forms of integration.

 - teamspace as a pas group provider
 - a pas group provider as a teamspace provider

the second form is rather difficult within the pas group interface as there
is no mechanism for group management or membership iteration, hence it
isn't worthwile. teamspace relying on external membership providers has had
some discussion on the teamspace mailing list which is recommended reading
for interested parties.

as regards property aspects, teamspace provides facilities for per
team user properties based on archetypes schema. however in the context
of the propertysheets specified by pas this is limited, and the format
isn't extensible. [xxx bring this up on the pas list] currently the best
that can be done is to do a schema map from at onto the pas propertysheets
which allows string, int, float, long, lines, and date properties.

another interesting integration with pas is providing teamspace as a role
plugin, in this capacity teamspace can act as a global provider of roles.
global provision isn't interesting in and of itself, however its possible
with a request context to limit the role provision to the context object,
thus allowing for non teamspace containment team security provisions.

Authors: kapil thangavelu
Created: 1/18/2004
$Id: pas.py 20162 2005-01-20 06:05:15Z hazmat $
"""

from sets import Set

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass 

from Products.PluggableAuthService.interface.plugins import \
     IGroupEnumerationPlugin, IGroupsPlugin, IRolesPlugin
from Products.PluggableAuthService.plugins.BasePlugin import BasePlugin

from Products.CMFCore.utils import getToolByName

from config import ROLE_MARKER

class TeamSpaceRolePlugin( BasePlugin ):
    """
    Provide TeamSpace team roles in the context of objects
    exhibiting an attribute marker for the current user.
    """
    security = ClassSecurityInfo()

    __implements__ = ( IRolesPlugin, )

    def __init__(self, id, title=''):
        self._setId( id )
        self.title = title

    def getRolesForPrincipal( self, principal, request=None ):
        """
        see IRolesPlugin.getRolesForPrincipal
        """
        if request is None:
            return ()
        
        published = request.get('PUBLISHED')
        role_teams = getattr(published, ROLE_MARKER, None)
        
        if not role_teams:
            return ()

        portal_teams = getToolByName(self, 'portal_teams')

        pid = principal.getId()

        roles = []
        memberships = portal_teams.getTeamMembershipsFor( pid, active=True)
        
        # all team roles in affect
        if role_teams is True:
            [roles.extend(mship.getTeamRoles()) for mship in in memberships]
            return tuple(Set( roles ))
        elif not isinstance( role_teams, (tuple, list)):
            return ()
        
        # only specific teams operating in this object context
        for m in memberships:
            if m.getTeam().getId() in role_teams:
                roles.extend( mship.getTeamRoles())
        return tuple(Set( roles ))


InitializeClass( TeamSpaceRolePlugin )

class TeamSpaceGroupPlugin( BasePlugin ):
    """
    Provide User Groups Based on TeamSpace memberships
    """

    security = ClassSecurityInfo()

    __implements__ = ( IGroupsPlugin, IGroupEnumerationPlugin )


    def __init__(self, id, title=''):
        self._setId( id )
        self.title = title

    def getGroupsForPrincipal( self, principal, request=None):
        """
        see IGroupsPlugin.getGroupsForPrincipal
        """
        portal_teams = getToolByName(self, 'portal_teams')
        teams = portal_teams.getTeamsByMemberId( principal.getId(),
                                                 active=True )
        return [t.getId() for t in teams]
        
    def enumerateGroups( self,
                         id = None,
                         exact_match = False,
                         sort_by = None,
                         max_results = None,
                         **kw ):
        """
        see IGroupEnumerationPlugin.enumerateGroups
        """
        portal_teams = getToolByName(self, 'portal_teams')
        teams = portal_teams.getTeams()
        filter = TeamFilter( id, exact_match, **kw )
        return [ self.getTeamInfo(team) for team if filter(teams)]
        
    security.declarePrivate('getTeamInfo')
    def getTeamInfo( self, team ):
        url = team.absolute_url()
        return {
            'id' : team.getId(),
            'pluginid' : self.getId(),
            'members_url' : url + '/ts_team_manage_memberships',
            'properties_url' : url,
            'title': team.Title()
            }

InitializeClass( TeamSpaceGroupPlugin )


class TeamFilter( object ):

    def __init__(self,  id, exact_match, **kw):
        if isinstance( id, str):
            id = [ id ]
        self.team_ids = id
        self.exact_match = not not exact_match
    def __call__(self, team):
        tid = team.getId()
        if self.exact_match:
            if tid in self.team_ids:
                return True
            return False
        for value in self.team_ids:
            if value.find( tid ) >= 0:
                return True

    
